$NOMOD51
$INCLUDE (reg51.inc)

; ***** BEGIN LICENSE BLOCK *****
; Version: MPL 1.1
;
; The contents of this file are subject to the Mozilla Public License Version 
; 1.1 (the "License"); you may not use this file except in compliance with 
; the License. You may obtain a copy of the License at 
; http://www.mozilla.org/MPL/
;
; Software distributed under the License is distributed on an "AS IS" basis,
; WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
; for the specific language governing rights and limitations under the
; License.
;
; The Original Code is firmware for the MiniDMX device.
;
; The Initial Developer of the Original Code is
; Tomsoft.
; Portions created by Tomsoft are Copyright (C) 2009
; Tomsoft. All Rights Reserved.
;
; Contributor(s):
;   Tommy Sools <tom@sools.com>
;
; ***** END LICENSE BLOCK *****

; MINIDMX V1.1 for AT89C4051

; V1.1:
; service mode changed so that all switches will be reset after test loop
; improved addressing macro
; generalised strobo channel with the four switch channels

; V1.0:
; initial version

; 16 MHz quartz = 16000000 Hz
; 1 pulse = 0,0000000625 seconds
; 1 cycle = 0,0000000625 seconds * 12 pulses = 0,00000075 seconds = 0,00075 milliseconds
; 16 bit timer is being used, meaning every 65536 cycles it will overflow.
; 65536 cycles take 65536 * 0,00075 milliseconds = 49,152 milliseconds.
; The DMX signal can only be considered to be lost after 1 second = 1000 milliseconds =>
; only after 1000 milliseconds / 49,152 milliseconds = 20,345052083333333333333333333333 timer overflows ~ 21 timer overflows.

; Slot 1: power 1 setting

;   0 - 127 power off
; 128 - 255 power on

; Slot 2: power 2 setting

;   0 - 127 power off
; 128 - 255 power on

; Slot 3: power 3 setting

;   0 - 127 power off
; 128 - 255 power on

; Slot 4: power 4 setting

;   0 - 127 power off
; 128 - 255 power on

; Slot 5: strobo trigger setting

;   0 - 127 strobo trigger off
; 128 - 255 strobo trigger on

; Alternate for slot 5 (possible future improvement)

;   0 -  25 strobo disabled
;  26 - 200 continuous speed setting (1 up to 15 triggers per second)
; 230 - 255 single flash trigger

		NAME		minidmx

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_switch5		EQU	P3.1	; switch 5
_switch4		EQU	P3.2	; switch 4
_switch3		EQU	P3.3	; switch 3
_switch2		EQU	P3.4	; switch 2
_switch1		EQU	P3.5	; switch 1

_inaddressl 		EQU	P1     	; low order address setting through dipswitches
_inaddressh		EQU	P3.7   	; high order address setting
			       		; (1 = 1; 2 = 2; 3 = 4; 4 = 8; 5 = 16; 6 = 32; 7 = 64; 8 = 128; 9 = 256; 10 = terminator)
					; all dipswitches off before powering up = service mode until power down
					; all dipswitches off only after powering up = address 1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

dmxpointer		EQU	R0   		; DMX value pointer
dmxcounter		EQU	R1   		; local DMX table counter
signallostcountdown	EQU	R2
tempaddressh		EQU	R5
tempaddressl		EQU	R6
loopcountdown		EQU	R7

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

tablesize		EQU	7	; size of DMX table (break + start code + 5 DMX values)
signallostcountdowninit	EQU	21	; number of times the timer must overflow before the signal will be considered to be lost

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?STACK		SEGMENT	IDATA		; ?STACK goes into IDATA RAM.
		RSEG	?STACK		; switch to ?STACK segment.
		DS	20

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?BIT		SEGMENT	BIT		; segment for BIT RAM.
	   	RSEG	?BIT		; switch to ?BIT segment
dmxsignalcheck: 	DBIT	1	; DMX signal received bit
dmxsignal: 		DBIT	1	; DMX signal present bit

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

?DATA		SEGMENT	DATA		; segment for DATA RAM.
	   	RSEG	?DATA		; switch to this data segment
initaddressh:		DS	1		; initial processed high address byte
initaddressl:		DS	1		; initial processed low address byte
delay1:			DS	1
delay2:			DS	1
delay3:			DS	1
table:   		DS	tablesize	; reserve bytes for DMX table
addressh:		DS	1		; address high byte
addressl:		DS	1		; address low byte

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

%*DEFINE (ADDRESS_REFRESH) LOCAL addressh_enabled, address_not_zero, address_end (
		MOV	tempaddressh,#1
		JNB	_inaddressh,%addressh_enabled	; retrieve high address byte
		MOV	tempaddressh,#0
%addressh_enabled:
		MOV	A,_inaddressl        		; retrieve low address byte
		CPL	A
		MOV	tempaddressl,A
		JNZ	%address_not_zero		; check if low address byte is zero
		CJNE	tempaddressh,#0,%address_not_zero	; check if high address byte is zero
		; the full address is zero, so use address 1
		SJMP	%address_end

%address_not_zero:	; the full address is not zero
; Decrement low address byte by 1, so that the address set using the dipswitches will be one-based rather than zero-based.
; This way a zero setting for the address could be used for special purposes.
; For address 1 (dipswitch 1 enabled): initaddressh = 0; initaddressl = 0
; For address 2 (dipswitch 2 enabled): initaddressh = 0; initaddressl = 1
; For address 3 (dipswitch 1 & 2 enabled): initaddressh = 0; initaddressl = 2
; etc.
		CLR	C
		SUBB	A,#1
		MOV	tempaddressl,A
		MOV	A,tempaddressh
		SUBB	A,#0
		MOV	tempaddressh,A

%address_end:	MOV	initaddressh,tempaddressh
		MOV	initaddressl,tempaddressl
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  0			; absolute Segment at Address 0
		LJMP	start			; reset location (jump to start)

?CODE		SEGMENT	CODE
		RSEG	?CODE			; switch to this code segment

start:          MOV	SP,#?STACK-1   		; assign stack at beginning

init:		CLR	dmxsignal
		CLR	dmxsignalcheck

		MOV	signallostcountdown,#signallostcountdowninit

		; check if the full address is zero
		JNB	_inaddressh,address_not_zero
		MOV	A,_inaddressl
		CJNE	A,#255,address_not_zero

		; address is zero, so go into service mode
		LJMP	service_mode

address_not_zero:
		%ADDRESS_REFRESH

		MOV	dmxpointer,#table+tablesize	; initialize DMX value pointer
emptytable:	DEC	dmxpointer
		MOV	@dmxpointer,#0
		CJNE	dmxpointer,#table,emptytable

		MOV     SCON,#10001000B		; mode 2 (9-bit UART, fixed baud rate),
						; no automatic address recognition, serial reception disabled,
						; set send bit 8, unset receive bit 8,
						; unset transmit and receive interrupt flags

		SETB	PS			; set high priority for serial port interrupt
		SETB    ES	       		; enable serial interrupt

		ORL	TMOD,#0000001B		; set timer 0 to 16-bit timer

		SETB	ET0	       		; enable timer 0 interrupt

		SETB    EA	      		; enable global interrupts
		SETB	REN			; enable serial reception
		SETB	TR0			; enable timer 0

		;CLR	_ledpower		; set power LED

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

loop:
		%ADDRESS_REFRESH

		; check if any DMX signal is present at all
		MOV	A,signallostcountdown
		JZ	signallost

		; output switch 1 status according to DMX value 1
		MOV	dmxcounter,#table+2
		CJNE	@dmxcounter,#127,checkswitch1
checkswitch1:	JNC	enableswitch1	; if @dmxcounter is greater than or equal to #128, enable switch
disableswitch1:	SETB	_switch1
		SJMP	checkswitch1end
enableswitch1:	CLR	_switch1
checkswitch1end:
		
		; output switch 2 status according to DMX value 2
		MOV	dmxcounter,#table+3
		CJNE	@dmxcounter,#127,checkswitch2
checkswitch2:	JNC	enableswitch2	; if @dmxcounter is greater than or equal to #128, enable switch
disableswitch2:	SETB	_switch2
		SJMP	checkswitch2end
enableswitch2:	CLR	_switch2
checkswitch2end:
		
		; output switch 3 status according to DMX value 3
		MOV	dmxcounter,#table+4
		CJNE	@dmxcounter,#127,checkswitch3
checkswitch3:	JNC	enableswitch3	; if @dmxcounter is greater than or equal to #128, enable switch
disableswitch3:	SETB	_switch3
		SJMP	checkswitch3end
enableswitch3:	CLR	_switch3
checkswitch3end:
		
		; output switch 4 status according to DMX value 4
		MOV	dmxcounter,#table+5
		CJNE	@dmxcounter,#127,checkswitch4
checkswitch4:	JNC	enableswitch4	; if @dmxcounter is greater than or equal to #128, enable switch
disableswitch4:	SETB	_switch4
		SJMP	checkswitch4end
enableswitch4:	CLR	_switch4
checkswitch4end:
		
		; output switch 5 status according to DMX value 5
		MOV	dmxcounter,#table+6
		CJNE	@dmxcounter,#127,checkswitch5
checkswitch5:	JNC	enableswitch5		; if @dmxcounter is greater than or equal to #128, enable switch
disableswitch5:	SETB	_switch5
		SJMP	checkswitch5end
enableswitch5:	CLR	_switch5
checkswitch5end:
		
		SJMP	loop

signallost:	; disable all switches
		SETB	_switch1
		SETB	_switch2
		SETB	_switch3
		SETB	_switch4
		SETB	_switch5

		SJMP	loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		; approx. 500 millseconds delay ==> 666666,66666666666666666666666667 cycles are neccesary
		; total 716297 cycles ==> 537,22275 milliseconds
%*DEFINE (DELAY) LOCAL ldelay, ldelay2, ldelay3 (
		MOV	delay1,#255	; 2 cycles

%ldelay:	MOV	delay2,#255	; 2 cycles * 255 = 510

%ldelay2:	NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		NOP			; 1 cycle * 255 * 255 = 65025
		DJNZ	delay2,%ldelay2	; 2 cycles * 255 * 255 = 130050
		DJNZ	delay1,%ldelay	; 2 cycles * 255 = 510
)
%*DEFINE (TESTDIP (dipbit)) (
		JB	%dipbit,$		; wait for dipswitch to be switched on
		CLR	_switch1  		; set switch 1
		CLR	_switch2		; set switch 2
		CLR	_switch3		; set switch 3
		CLR	_switch4		; set switch 4
		CLR	_switch5		; set switch 5
		%DELAY
		SETB	_switch1  		; reset switch 1
		SETB	_switch2		; reset switch 2
		SETB	_switch3		; reset switch 3
		SETB	_switch4		; reset switch 4
		SETB	_switch5		; reset switch 5

		JNB	%dipbit,$		; wait for dipswitch to be switched off
		CLR	_switch1  		; set switch 1
		CLR	_switch2		; set switch 2
		CLR	_switch3		; set switch 3
		CLR	_switch4		; set switch 4
		CLR	_switch5		; set switch 5
		%DELAY
		SETB	_switch1  		; reset switch 1
		SETB	_switch2		; reset switch 2
		SETB	_switch3		; reset switch 3
		SETB	_switch4		; reset switch 4
		SETB	_switch5		; reset switch 5
)

service_mode:	; slight delay before testing switchs
		%DELAY

		; running light on switches
		MOV	loopcountdown,#9
service_running_light:
		%DELAY
		SETB	_switch5  		; reset switch 5
		CLR	_switch1  		; set switch 1
		%DELAY
		SETB	_switch1  		; reset switch 1
		CLR	_switch2  		; set switch 2
		%DELAY
		SETB	_switch2 		; reset switch 2
		CLR	_switch3 		; set switch 3
		%DELAY
		SETB	_switch3 		; reset switch 3
		CLR	_switch4 		; set switch 4
		%DELAY
		SETB	_switch4		; reset switch 4
		CLR	_switch5		; set switch 5
		DJNZ	loopcountdown,service_running_light

		SETB	_switch5		; reset switch 5

		; test dipswitches. the user will have to switch the dipswitches on and off one after another,
		; starting with dipswitch 1 and ending with dipswitch 9.
		%TESTDIP (_inaddressl.0)
		%TESTDIP (_inaddressl.1)
		%TESTDIP (_inaddressl.2)
		%TESTDIP (_inaddressl.3)
		%TESTDIP (_inaddressl.4)
		%TESTDIP (_inaddressl.5)
		%TESTDIP (_inaddressl.6)
		%TESTDIP (_inaddressl.7)
		%TESTDIP (_inaddressh)

service_init:
service_loop:	; allow switches to be controlled by dipswitches

		JB	_inaddressl.0,service_disableswitch1
service_enableswitch1:	CLR	_switch1
		SJMP	service_switch1end
service_disableswitch1:	SETB	_switch1
service_switch1end:

		JB	_inaddressl.1,service_disableswitch2
service_enableswitch2:	CLR	_switch2
		SJMP	service_switch2end
service_disableswitch2:	SETB	_switch2
service_switch2end:

		JB	_inaddressl.2,service_disableswitch3
service_enableswitch3:	CLR	_switch3
		SJMP	service_switch3end
service_disableswitch3:	SETB	_switch3
service_switch3end:

		JB	_inaddressl.3,service_disableswitch4
service_enableswitch4:	CLR	_switch4
		SJMP	service_switch4end
service_disableswitch4:	SETB	_switch4
service_switch4end:

		JB	_inaddressl.4,service_disableswitch5
service_enableswitch5:	CLR	_switch5
		SJMP	service_switch5end
service_disableswitch5:	SETB	_switch5
service_switch5end:

		SJMP	service_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  0BH	      		; 0BH is address for Timer 0 interrupt			;!3 cycles this far
		LJMP	timer0int							     		;!2 cycles

?TIMER0		SEGMENT	CODE
		RSEG	?TIMER0

timer0int:	PUSH	PSW	     		; store PSW for safekeeping				;!2 cycles
		PUSH	ACC	      		; store ACC for safekeeping				;!2 cycles

		MOV	C,dmxsignalcheck
		MOV	dmxsignal,C
		CLR	dmxsignalcheck
		JC	blinkpowerled
enablepowerled:	;CLR	_ledpower		; set power LED to indicate that there is no DMX signal going on
		MOV	A,signallostcountdown
		JZ	checkpowerledend
		DEC	signallostcountdown
		SJMP	checkpowerledend
blinkpowerled:	MOV	signallostcountdown,#signallostcountdowninit
		;CPL	_ledpower		; reverse power LED's status for DMX signal indication
checkpowerledend:

timer0int_end:	POP	ACC	      		; restore ACC						;!2 cycles
		POP	PSW	      		; restore PSW						;!2 cycles
		RETI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		CSEG	AT  23H	     			; 23H is the address for Serial interrupt		; 3 cycles this far
;		LJMP	serialint											; 2 cycles

;?SERIAL		SEGMENT	CODE
;		RSEG	?SERIAL

serialint:	PUSH	PSW	      			; store PSW for safekeeping				; 2 cycles
		PUSH	ACC	     			; store ACC for safekeeping				; 2 cycle

                JNB	RI,serialint_end     									; 2 cycles

		SETB	dmxsignalcheck										; 1 cycle

                JNB	RB8,dmxbreak										; 2 cycles

		; if counter is zero then skip
		CJNE	dmxpointer,#table,not_zero								; 2 cycles
		SJMP	serialint_end										; 2 cycles
		
dmxbreak: 	; break detected
		MOV	dmxpointer,#table+1						     			; 1 cycle

		MOV	addressh,initaddressh		; retrieve high order address byte			; 2 cycles
		MOV	addressl,initaddressl		; retrieve low order address byte			; 2 cycles

		SJMP	serialint_end								       		; 2 cycles

not_zero:      	; if counter exceeds the table's size then skip
		CJNE	dmxpointer,#table+tablesize,dmxlimitcheck	      					; 2 cycles

dmxlimitcheck:	JNC	serialint_end										; 2 cycles

		CJNE	dmxpointer,#table+1,nostartbyte	; check if this is a start byte being received		; 2 cycles

		; this is a start byte being received
		MOV	A,SBUF				; retrieve start byte					; 1 cycle
		JZ	goodstartbyte                   ; the received start byte is zero			; 2 cycles

badstartbyte:	MOV	dmxpointer,#table		; the received start byte is not zero, reset pointer	; 1 cycle
		SJMP	serialint_end										; 2 cycles

goodstartbyte:	INC	dmxpointer								       		; 1 cycle
		SJMP	serialint_end

nostartbyte:	MOV	A,addressh										; 1 cycle
		JNZ	decrement										; 2 cycles
		MOV	A,addressl										; 1 cycle
		JNZ	decrement										; 2 cycles

		MOV	@dmxpointer,SBUF		; store DMX value					; 2 cycles

		INC	dmxpointer								       		; 1 cycle
		SJMP	serialint_end			; counter is already zero, so don't decrement again	; 2 cycles

decrement:	CLR	C											; 1 cycle
		MOV	A,addressl										; 1 cycle
		SUBB	A,#1											; 1 cycle
		MOV	addressl,A										; 1 cycle
		MOV	A,addressh										; 1 cycle
		SUBB	A,#0	     										; 1 cycle
		MOV	addressh,A								       		; 1 cycle

serialint_end:	CLR	RI				; serial receive data handled		      		; 1 cycle
		POP	ACC				; restore ACC				      		; 2 cycles
		POP	PSW				; restore PSW				      		; 2 cycles
		RETI								       				; 2 cycles

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		END			; End Of File

